package Catalyst::Middleware::AccessLog::Hook;
$Catalyst::Middleware::AccessLog::Hook::VERSION = '0.201410051';

use strict;
use base 'Plack::Middleware';

use Plack::Middleware::AxsLog;
use Plack::Util::Accessor qw(logger);

*call = \&Plack::Middleware::AxsLog::call;

sub log_line {
    my $self = shift;

    $self->logger->(@_);
}

1;

__END__

=pod

=head1 NAME

Catalyst::Middleware::AccessLog::Hook

=head1 VERSION

version 0.201410051

=head1 SYNOPSIS

    package MyWebApp;
    use Moose;
    extends qw(Catalyst);

    __PACKAGE__->config(
        psgi_middleware => [
            '+Catalyst::Middleware::AccessLog::Hook' => {
                logger => sub {
                    # Plack::Middleware::AxsLog::log_line と同じ引数
                    my ($t0, $env, $res, $length) = @_;
                    my $stash  = Catalyst::Middleware::Stash::get_stash($env)->();

                    my $hash = $c->stash->{accesslog_fields} || {};
                    $hash->{size}        //= $length;
                    $hash->{log_elapsed} //= sprintf '%f', Time::HiRes::tv_interval($t0);

                    # syslogへ
                    openlog($hash->{app_name}, "pid", "local2");
                    syslog("info", encode_json($hash));

                    # RDBへも
                    container('schema')->resultset('AccessLog')->create($hash);
                },
            },
        ],
        ...
    );

    before finalize => sub {
        my $c = shift;

        my $f = $c->stash->{accesslog_fields} ||= {};

        # Apache風
        # LogFormat "%v %h %l %u %t "%r" %>s %b "%{Referer}i" "%{User-Agent}i"" combined
        # format /^(?<vhost>[^ ]*) (?<host>[^ ]*) [^ ]* (?<user>[^ ]*) \[(?<time>[^\]]*)\] "(?<method>\S+)(?: +(?<path>[^ ]*) +\S*)?" (?<code>[^ ]*) (?<size>[^ ]*)(?: "(?<referer>[^\"]*)" "(?<agent>[^\"]*)")?$/
        $f->{vhost}          = $c->req->base->host_port;
        $f->{host}           = $c->req->address;
        $f->{user}           = $c->req->remote_user;
        $f->{path}           = do {
            my $path  = $c->req->uri->path;
            my $query = $c->req->uri->query;
            if (defined $query && length $query) {
                $query = "?$query";
            } else {
                $query = "";
            }
            "${path}${query}";
        };
        $f->{time}           = POSIX::strftime('%Y-%m-%d %H:%M:%S', localtime(time));
        $f->{method}         = $c->req->method;
        $f->{code}           = $c->res->code;
        $f->{referer}        = $c->req->referer;
        $f->{agent}          = $c->req->user_agent;

        # その他
        $f->{app_name}       = 'mywebapp_dev';
        $f->{hostname}       = $c->req->hostname;
        $f->{app_elapsed}    = sprintf '%f', $c->stats->elapsed if $c->stats->enable;
        $f->{user_id}        = $c->user->$_call_if_object('id');
        $f->{username}       = $c->user->$_call_if_object('username');
        $f->{session_id}     = $c->sessionid;
        $f->{mobile_user_id} = $c->req->mobile_agent->user_id;
        $f->{unique_id_id}   = $c->req->header('X-UNIQUE-ID');
    };

=head1 MOTIVATION

    Catalyst::Plugin::AccessLog の拡張で Catalyst::Plugin::AccessLog::BOKUTIN を使っていたが
    Catalyst が Plack::Middleware::ContentLength を使うようになってから
    after finalize 時では $c->res->content_length が拾えなくなった。

    ログの記録を content-length の拾える Plack::Middleware::AxsLog と同等のタイミングで行ないたい。

    $c から取得できる値も記録したい。

=head1 AUTHOR

Tomohiro Hosaka <bokutin@bokut.in>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Tomohiro Hosaka.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
